@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2017, University of Amsterdam
+    Copyright (c)  2011-2018, University of Amsterdam
                               VU University Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -34,6 +35,7 @@
 */
 
 #ifdef __WINDOWS__
+#define _WIN32_WINNT 0x0600		/* Vista */
 #include "windows/uxnt.h"
 #ifdef WIN64
 #include "config/win64.h"
@@ -195,6 +197,7 @@ S__setbuf(IOSTREAM *s, char *buffer, size_t size)
   } else
   { if ( !(newunbuf = malloc(size+UNDO_SIZE)) )
     { errno = ENOMEM;
+      S__seterror(s);
       return -1;
     }
     newflags &= ~SIO_USERBUF;
@@ -230,7 +233,8 @@ S__setbuf(IOSTREAM *s, char *buffer, size_t size)
       }
     }
 
-    memcpy(newbuf, s->bufp, copy);
+    if ( copy > 0 )
+      memcpy(newbuf, s->bufp, copy);
     S__removebuf(s);
     s->unbuffer = newunbuf;
     s->bufp = s->buffer = newbuf;
@@ -439,7 +443,6 @@ S__wait(IOSTREAM *s)
     if ( rc < 0 && errno == EINTR )
     { if ( PL_handle_signals() < 0 )
       { Sset_exception(s, PL_exception(0));
-	s->flags |= SIO_CLEARERR;
 	errno = EPLEXCEPTION;
 	return -1;
       }
@@ -524,11 +527,10 @@ S__flushbuf(IOSTREAM *s)
     { if ( errno == EINTR )
       { if ( PL_handle_signals() < 0 )
 	{ Sset_exception(s, PL_exception(0));
-	  s->flags |= SIO_CLEARERR;
 	  errno = EPLEXCEPTION;
 	} else
 	  goto retry;
-      } else if ( errno != EPLEXCEPTION && !(s->flags&SIO_NOERROR) )
+      } else
 	S__seterror(s);
       rc = -1;
       goto out;
@@ -675,8 +677,7 @@ S__fillbuf(IOSTREAM *s)
 	return -1;
 #endif
       } else
-      { if ( !(s->flags & SIO_FERR) )	/* error already set */
-	  S__seterror(s);
+      {	S__seterror(s);
 	return -1;
       }
     }
@@ -720,14 +721,6 @@ update_linepos(IOSTREAM *s, int c)
 
 
 
-int
-S__fcheckpasteeof(IOSTREAM *s, int c)
-{ S__checkpasteeof(s, c);
-
-  return c;
-}
-
-
 int
 S__fupdatefilepos_getc(IOSTREAM *s, int c)
 { IOPOS *p = s->position;
@@ -750,7 +743,6 @@ S__updatefilepos(IOSTREAM *s, int c)
   { update_linepos(s, c);
     p->charno++;
   }
-  S__checkpasteeof(s,c);
 
   return c;
 }
@@ -860,7 +852,7 @@ reperror(int c, IOSTREAM *s)
     return c;
   }
 
-  Sseterr(s, SIO_FERR|SIO_CLEARERR, "Encoding cannot represent character");
+  Sseterr(s, SIO_FERR, "Encoding cannot represent character");
   return -1;
 }
 
@@ -1350,7 +1342,7 @@ Sread_pending(IOSTREAM *s, char *buf, size_t limit, int flags)
   { int c = S__fillbuf(s);
 
     if ( c < 0 )
-    { if ( (s->flags & SIO_FEOF) )
+    { if ( (s->flags & SIO_FEOF) && Sfpasteof(s) != TRUE )
 	return 0;
       return c;
     }
@@ -1453,8 +1445,7 @@ ScheckBOM(IOSTREAM *s)
       return 0;
 
     if ( S__fillbuf(s) == -1 )
-    { if ( s->limitp - s->bufp > 0 )
-	s->flags &= ~SIO_FEOF;
+    { s->flags &= ~SIO_FEOF;
       return 0;				/* empty stream */
     }
     s->bufp--;
@@ -1508,7 +1499,10 @@ Sfeof(IOSTREAM *s)
 
 static int
 S__seterror(IOSTREAM *s)
-{ s->io_errno = errno;
+{ if ( (s->flags & SIO_FERR) )
+    return 0;				/* error already set */
+
+  s->io_errno = errno;
 
   if ( !(s->flags&SIO_CLOSING) &&	/* s->handle is already invalid */
        s->functions->control )
@@ -1548,7 +1542,7 @@ Sfpasteof(IOSTREAM *s)
 
 
 #define SIO_ERROR_FLAGS (SIO_FEOF|SIO_WARN|SIO_FERR| \
-			 SIO_FEOF2|SIO_TIMEOUT|SIO_CLEARERR)
+			 SIO_FEOF2|SIO_TIMEOUT)
 
 void
 Sclearerr(IOSTREAM *s)
@@ -1568,7 +1562,7 @@ Sclearerr(IOSTREAM *s)
 int
 Sseterr(IOSTREAM *s, int flags, const char *message)
 { for(; s && s->magic == SIO_MAGIC; s = s->upstream )
-  { s->flags = (s->flags & ~(SIO_WARN|SIO_FERR|SIO_CLEARERR)) | flags;
+  { s->flags = (s->flags & ~(SIO_WARN|SIO_FERR)) | flags;
 
     if ( s->message )
     { free(s->message);
@@ -2111,29 +2105,6 @@ Svprintf(const char *fm, va_list args)
 #define A_LEFT	0			/* left-aligned field */
 #define A_RIGHT 1			/* right-aligned field */
 
-#ifdef __WINDOWS__
-/* This is a copy from src/pl-nt.c, copied here to avoid dependency
-   of pl-stream.c on SWI-Prolog internals.
-*/
-
-static int
-ms_snprintf(char *buffer, size_t count, const char *fmt, ...)
-{ va_list ap;
-  int ret;
-
-  va_start(ap, fmt);
-  ret = _vsnprintf(buffer, count-1, fmt, ap);
-  va_end(ap);
-
-  if ( ret < 0 || ret == count )
-  { ret = (int)count;
-    buffer[count-1] = '\0';
-  }
-
-  return ret;
-}
-#endif
-
 #define SNPRINTF3(fm, a1) \
 	{ size_t __r; \
 	  assert(fs == fbuf); \
@@ -3657,7 +3628,7 @@ Sopenmem(char **buffer, size_t *sizep, const char* mode)
     Mode is "r" or "w".  The mode "rF" calls PL_free(*buffer) at when
     closed.
 
-Note: Its is NOT allows to access   streams  created with this call from
+Note: Its is NOT allowed to access   streams created with this call from
 multiple threads. This is ok for all   usage inside Prolog itself (often
 through tellString()/toldString(). This call is   intented  to use write
 and other output predicates to create strings.
