@@ -782,6 +782,7 @@ rec_error(CompileInfo info)
     }
     default:
       assert(0);
+      return FALSE;
   }
 }
 
@@ -1245,17 +1246,14 @@ fetchSizeInt(CopyInfo b)
 
 static int64_t
 fetchInt64(CopyInfo b)
-{ int64_t val = 0;
+{ uint64_t val = 0;
   uint bytes = *b->data++;
-  uint shift = (sizeof(int64_t)-bytes)*8;
+  uint64_t sign = 1ULL << (bytes * 8 - 1);
 
   while(bytes-- > 0)
     val = (val << 8) | (*b->data++ & 0xff);
 
-  val <<= shift;
-  val >>= shift;
-
-  return val;
+  return (int64_t)((val ^ sign) - sign);
 }
 
 
@@ -1490,6 +1488,7 @@ copy_record(Word p, CopyInfo b ARG_LD)
 	    name = ATOM_dict;
 	    break;
 	  default:
+	    name = 0;
 	    assert(0);
 	}
 
@@ -1749,6 +1748,14 @@ markAtomsRecord(Record record)
 }
 
 
+#ifdef O_DEBUG_ATOMGC
+void
+unregister_atom_rec(atom_t a)
+{ PL_unregister_atom(a);
+}
+#endif
+
+
 bool
 freeRecord(Record record)
 { if ( true(record, R_DUPLICATE) && --record->references > 0 )
@@ -1762,7 +1769,11 @@ freeRecord(Record record)
 
     ci.base = ci.data = dataRecord(record);
     ci.version_map = NULL;
+#ifdef O_DEBUG_ATOMGC
+    scanAtomsRecord(&ci, unregister_atom_rec);
+#else
     scanAtomsRecord(&ci, PL_unregister_atom);
+#endif
     assert(ci.data == addPointer(record, record->size));
   }
 #endif
@@ -1847,6 +1858,7 @@ PL_recorded_external(const char *rec, term_t t)
 	  fetchAtomW(&b, &a);
 	  break;
 	default:
+	  a = 0;
 	  assert(0);
       }
       rc = PL_put_atom(t, a);
@@ -1966,8 +1978,13 @@ PRED_IMPL("current_key", 1, current_key, PL_FA_NONDETERMINISTIC)
 
     while(advanceTableEnum(e, &sk, &sv))
     { RecordList rl = sv;
+      RecordRef record;
 
-      if ( rl->firstRecord && unifyKey(A1, rl->key) )
+      PL_LOCK(L_RECORD);
+      record = firstRecordRecordList(rl);
+      PL_UNLOCK(L_RECORD);
+
+      if ( record && unifyKey(A1, rl->key) )
       { PL_close_foreign_frame(fid);
 
 	ForeignRedoPtr(e);
